# This function provides the interface for users to select/input
# arguments for KNN
#
# items - a vector of character strings for the items to be picked
# from.
#
# Copyright 2002 J. Zhang, all rights reserved
#
KNNArgsWidget <- function (){

    args2Return <- list()

    columns <- NULL
    colIndex <- NULL
    indexInSel <- NULL
    text2 <- "Select item(s) from the list box on the left"
    targetTag <- tclVar()
    normValue <- tclVar()
    kValue <- tclVar()
    distValue <- tclVar()
    transValue <- tclVar()

    end <- function(){
        if(nchar(tclvalue(targetTag)) != 10){
            temp <- tkmessageBox(title = "Incorrect tag",
                                     message = paste("A target tag has",
                                     "to be set and of 10 characters"),
                                     icon = "warning", type = "ok")
        }else{
            args2Return[["targetSAGE"]] <<- toupper(tclvalue(targetTag))
            args2Return[["normalize"]] <<- tclvalue(normValue)
            if(length(columns) != 0 ){
                args2Return[["libs"]] <<- unique(names(libs[match(columns,
                                                                  libs)]))
            }else{
                args2Return[["libs"]] <<- "*"
            }
            args2Return[["k"]] <<- as.numeric(tclvalue(kValue))
            args2Return[["dist"]] <<- tclvalue(distValue)
            args2Return[["trans"]] <<- tclvalue(transValue)

            tkdestroy(base)
        }
    }
    cancel <- function(){
        args2Return <<- NULL
        tkdestroy(base)
    }

    # When a user double clicks a column name in the list box, put the
    # library name in the vector for selected SAGE libraries.
    colDClicked <- function(){
        column <- as.character(tkget(colView,(tkcurselection(colView))))
        columns <<- unique(c(columns, column))
        writeList(selectView, columns)
        tkconfigure(clearBut, state = "normal")
        tkconfigure(selectBut, state = "disabled")
    }
    # When a user single clicked a column name, remember that name and
    # activate the select button
    colSClicked <- function(){
        colIndex <<- unlist(strsplit(
                              as.character(tkcurselection(colView)), " "))
        tkconfigure(selectBut, state = "normal")
    }
    # When a user click the select button, put the selected column name
    # in the list box for selected columns
    selectCol <- function(){
        for(i in colIndex){
            columns <<- c(columns, as.character(tkget(colView, i)))
        }
        writeList(selectView, unique(columns))
        tkconfigure(selectBut, state = "disabled")
        tkconfigure(clearBut, state = "normal")
    }
    # Remember the column name when a name in the selected column names list
    # box is single clicked
    selSClick <- function(){
        indexInSel <<- unlist(strsplit(
                              as.character(tkcurselection(selectView)), " "))
        tkconfigure(remBut, state = "normal")
    }
    # Removes the selected column name from the list box for selected
    # column names
    dropSelCol <- function(){
        for(i in indexInSel){
            columns <<- columns[columns != as.character(tkget(selectView, i))]
        }
        writeList(selectView, unique(columns))
        tkconfigure(remBut, state = "disabled")
    }
    # Remove everything from the list box for selected column names
    clearSelCol <- function(){
        writeList(selectView, "")
        columns <<- NULL
        tkconfigure(clearBut, state = "disabled")
    }
    # Select all the libraries
    selectAllLibs <- function(){
        columns <<- as.vector(libs)
        writeList(selectView, columns, clear = TRUE)
        tkconfigure(clearBut, state = "normal")
    }
    # Get library column names from DB
    libs <- queryInfoDB(infoCol = "filename")
    # Get saved argument values to put in the corresponding widgets
    SAGELyzerArgs <- getSLyzerArgs()

    base <- tktoplevel()
    on.exit(tkdestroy(base))
    tktitle(base) <- "BioC SAGElyzer KNN"

    tkpack(tklabel(base, text = "SAGElyzer KNN arguments"), expand = FALSE,
           fill = "x", padx = 5, pady = 5)
    tagFrame <- tkframe(base)
    tkpack(tklabel(tagFrame, text = "Target tag"), side = "left",
           expand = FALSE, fill = "x")
    tagEntry <- tkentry(tagFrame, width = 15, textvariable = targetTag)
    tkpack(tagEntry, side = "left", expand = TRUE, fill = "x")
    if(SAGELyzerArgs[["targetSAGE"]] != ""){
        writeList(tagEntry, SAGELyzerArgs[["targetSAGE"]], clear = TRUE)
    }
    tkpack(tagFrame, expand = FALSE, fill = "x", padx = 5, pady = 5)

    argFrame <- tkframe(base)
    normLabel <- tklabel(argFrame, text = "Normalization")
    kLabel <- tklabel(argFrame, text = "k value")
    distLabel <- tklabel(argFrame, text = "Distance")
    transLabel <- tklabel(argFrame, text = "Trnsformation")
    normFrame <- tkframe(argFrame)
    kFrame <- tkframe(argFrame)
    distFrame <- tkframe(argFrame)
    transFrame <- tkframe(argFrame)
    dropdownList(normFrame, c("min", "max", "none"),
                              normValue, 15, SAGELyzerArgs[["normalize"]])
    dropdownList(kFrame, c(50, 100, 150, 200, 250, 300,
                                       400, 500, 1000, 2000), kValue,
                           15, SAGELyzerArgs[["k"]])
    dropdownList(distFrame, c("euclidean", "maximum", "manhattan",
                 "canberra", "correlation","binary"), distValue,
                              15, SAGELyzerArgs[["dist"]])
    dropdownList(transFrame, c("sqrt", "log10", "log2", "exp"),
                               transValue, 15, SAGELyzerArgs[["trans"]])
    tkgrid(normLabel, normFrame, kLabel, kFrame, padx = 5, pady = 4)
    tkgrid(distLabel, distFrame, transLabel, transFrame, padx = 5,
           pady = 4)
    tkgrid.configure(normLabel, kLabel, distLabel, transLabel, sticky = "e")
    tkgrid.configure(normFrame, kFrame, distFrame, transFrame, sticky = "w")
    tkpack(argFrame, expand = FALSE, fill = "x")

    # Lists for column names
    midFrame <- tkframe(base)
    text2Label <- tklabel(midFrame, text = text2, font = "Helvetica 12")
    tkgrid(text2Label, columnspan = 2, pady = 2)
    # Label for available SAGE libs
    leftFrame <- tkframe(midFrame)
    tkgrid(tklabel(leftFrame, text = "SAGE libraries",
                   font = "Helvetica 11"), columnspan = 2)
    # List box showing the available SAGE libs
    colFrame <- tkframe(leftFrame)
    colView <- makeViewer(colFrame, vWidth = 40, vHeight = 15,
                           hScroll = TRUE)
    tkbind(colView, "<Double-Button-1>", colDClicked)
    tkbind(colView, "<B1-ButtonRelease>", colSClicked)
    tkgrid(colFrame, columnspan = 2)
    selectAllBut <- tkbutton(leftFrame, text = "Select all >>", width = 12,
                             command = selectAllLibs)
    selectBut <- tkbutton(leftFrame, text = "Select >>", width = 12,
		      state = "disabled", command = selectCol)
    tkgrid(selectAllBut, selectBut, padx = 5)
    tkgrid.configure(selectAllBut, sticky = "e")
    tkgrid.configure(selectBut, sticky = "w")
    tkconfigure(colView, selectmode = "extended")
    # Put the list box for selected SAGE libs and the associated buttons
    rightFrame <- tkframe(base)
    label2 <- tklabel(rightFrame, text = "SAGE libraries picked",
                        font = "Helvetica 11")
    tkgrid(label2, columnspan = 2)
    selLFrame <- tkframe(rightFrame)
    selectView <- makeViewer(selLFrame, vWidth = 40, vHeight = 15,
                             hScroll = TRUE)
    if(length(SAGELyzerArgs[["libs"]]) > 1 ||
                                   SAGELyzerArgs[["libs"]] != "*"){
        columns <- libs[match(SAGELyzerArgs[["libs"]], names(libs))]
        columns <- unique(as.vector(columns))
        writeList(selectView, columns, clear = TRUE)
    }
    tkgrid(selLFrame, columnspan = 2, padx = 5)
    tkconfigure(selectView, selectmode = "extended")
    tkbind(selectView, "<B1-ButtonRelease>", selSClick)
    remBut <- tkbutton(rightFrame, text = "<< Remove", width = 12,
		      state = "disabled", command = dropSelCol)
    clearBut <- tkbutton(rightFrame, text = "Clear", width = 12,
		      state = "disabled", command = clearSelCol)
    tkgrid(remBut, clearBut, padx = 5)
    tkgrid.configure(remBut, sticky = "e")
    tkgrid.configure(clearBut, sticky = "w")

    tkgrid(leftFrame, rightFrame)
    tkpack(midFrame)

    # Put the end buttons
    butFrame <- tkframe(base)
    endBut <- tkbutton(butFrame, text = "Continue", width = 12,
                       command = end)
    cancelBut <- tkbutton(butFrame, text = "Cancel", width = 12,
                          command = cancel)
    tkgrid(endBut, cancelBut, padx = 10)
    tkpack(butFrame, pady = 5)

    writeList(colView, as.vector(libs))

    tkwait.window(base)

    return(args2Return)
}









