# This widget is the main widget of SAGElyzer
#
# Copyright 2003, J. Zhang. All rights reserved.
#

SAGEWidget <- function(){

    DBSET <- FALSE

    makeCon <- function(){
        if(getDBArgs()){
            if(!DBSET){
                popTask()
            }
        }
    }

    popTask <- function(){
        DBSET <<- TRUE

        taskButs <- list()
        tasks <- getTasks()
        tkconfigure(taskList, state = "normal")
        for(task in 1:length(tasks)){
            fun <- function() {}
            body <- list(as.name("{"),
                         substitute(tasks[[i]](base, funcList, status),
                                    list(i = task)))
            body(fun) <- as.call(body)
            assign(paste("taskFun", task, sep = ""), fun)
            taskButs[[task]] <- tkbutton(base, width = 16, text =
                                         names(tasks)[task])
            tkbind(taskButs[[task]], "<B1-ButtonRelease>",
                   get(paste("taskFun", task, sep = "")))
            ## bind to tooltip
            tip <- function() {}
            body <- list(as.name("{"),
                         substitute(tooltip(i, taskButs[[j]]),
                                    list(i = getTaskTips(names(tasks)[task]),
                                         j = task)))
            body(tip) <- as.call(body)
#            assign(paste("tipFun", task, sep = ""), tip)
            tkbind(taskButs[[task]], "<Enter>", tip)
#                   get(paste("tipFun", task, sep = "")))
            tkwindow.create(taskList, "end",
                            window = taskButs[[task]])
            tkinsert(taskList, "end", "\n")
        }
        tkconfigure(taskList, state = "disabled")
    }

    quit <- function(){
        tkdestroy(base)
    }

    base <- tktoplevel()
    on.exit(tkdestroy(base))

    tktitle(base) <- "SAGElyzer"
    tkpack(tklabel(base, text = paste("Welcome to SAGElyzer,\n",
                         "Bioconductor's SAGE data management",
                         "and analysis tool")),
           expand = FALSE, fill = "x", pady = 10)
    mainFrame <- tkframe(base, borderwidth = 2, relief = "raised")
    dbFrame <- tkframe(mainFrame)
    tkpack(tklabel(dbFrame, text = "Connection to a database is required"),
           side = "left", expand = FALSE, fill = "x")
    dbBut <- tkbutton(dbFrame, text = "Connect", width = 8, command = makeCon)
    tkpack(dbBut, side = "left", expand = FALSE, fill = "x")
    tkpack(dbFrame, expand = FALSE, fill = "x", padx = 3, pady = 3)

    listFrame <- tkframe(mainFrame)
    taskFrame <- tkframe(listFrame)
    tkpack(tklabel(taskFrame, text = "Tasks"), expand = FALSE, fill = "x")
    # Tasks are jobs to be done (e. g. integrate data, do knn, ...)
    tasks <- getTasks()
    taskList <- makeViewer(taskFrame, vWidth = 20, vHeight = 18,
                           vScrol = TRUE, hScrol = FALSE, what = "text")
    tkconfigure(taskList, state = "disabled")
    tkpack(taskFrame, side = "left", expand = TRUE, fill = "both")
    funcFrame <- tkframe(listFrame)
    tkpack(tklabel(funcFrame, text = "Procedures"),
           expand = FALSE, fill = "x")
    funcList <- makeViewer(funcFrame, vWidth = 20, vHeight = 18,
                           vScrol = TRUE, hScrol = FALSE, what = "text")
    tkconfigure(funcList, state = "disabled")
    tkpack(funcFrame, side = "left", expand = TRUE, fill = "both")
    tkpack(listFrame, expand = TRUE, fill = "y")

    tkpack(mainFrame, expand = TRUE, fill = "both", padx = 5)

    endBut <- tkbutton(base, text = "Quit", width = 10, comman = quit)
    tkpack(endBut, expand = FALSE, padx = 5, pady = 5)

    status <- tkentry(base, state = "disabled", foreground = "red")
    tkpack(status, expand = FALSE, fill = "x")
    if(length(getSLyzerArgs()$dbArgs) > 1){
        popTask()
    }

    tkwait.window(base)

    return(invisible())
}


getTasks <- function(){
    return(list("Manage Data" = getDMProc, "knn" = getKNNProc))
}

getDMProc <- function(base, TBox, status){
    procs <- list("Get GEO SAGE" = SAGEFromGEO, "Integrate SAGE" =
                  procSAGE, "Map SAGE" = mapSAGE2UG)
    butsInTBox(base, TBox, status, procs, TRUE)
}

getKNNProc <- function(base, TBox, status){
    procs <- list("Set arguments" = setKNNArgs,
                  "Run knn" = runSLyzer,
                  "Get counts" = getLibCounts,
                  "Map SAGE" = linkTag2UG,
                  "Find neighbor genes" = findNG4Tag)
    butsInTBox(base, TBox, status, procs, TRUE)
}

butsInTBox <- function(base, TBox, status, butList, clear = FALSE){
    tkconfigure(TBox, state = "normal")
    if(clear){
        tkdelete(TBox, "0.0", "end")
    }
    procButs <- list()
    for(i in 1:length(butList)){
        fun <- function() {}
        body <- list(as.name("{"),
                     substitute(tkconfigure(status, state = "normal")),
                     substitute(writeList(status, paste("Running ",
                       "procedure \"", j, "\". Please wait.", sep = ""),
                       clear = TRUE), list(j = names(butList)[i])),
                     substitute(butList[[j]](), list(j = names(butList)[i])),
                     substitute(writeList(status, "", clear = TRUE)),
                     substitute(tkconfigure(status, state = "disabled")))
        body(fun) <- as.call(body)
        assign(paste("procFun", i, sep = ""), fun)
        procButs[[i]] <- tkbutton(base, width = 16, text = names(butList)[i])
        tkbind(procButs[[i]],  "<B1-ButtonRelease>",
               get(paste("procFun", i, sep = "")))

        tip <- function() {}
        body <- list(as.name("{"),
                     substitute(tooltip(j, procButs[[k]]),
                                list(j = getTaskTips(names(butList)[i]),
                                     k = i)))
        body(tip) <- as.call(body)
        assign(paste("tipFun", i, sep = ""), tip)
        tkbind(procButs[[i]], "<Enter>",
               get(paste("tipFun", i, sep = "")))

        tkwindow.create(TBox, "end", window = procButs[[i]])
        tkinsert(TBox, "end", "\n")
    }
    tkconfigure(TBox, state = "disabled")
}

getTaskTips <- function(task){
    load(file.path(.path.package("SAGElyzer"), "data", "SAGEToolTips.rda"))
    if(any(names(SAGEToolTips) == task)){
        return(SAGEToolTips[[task]])
    }else{
        stop("Unknown task name passed to getTaskTips")
    }
}

