# This function provides the interface for inputing query parameters
# for querying a table in a given database. Interface between R and
# the underlaying database management system is through Rdbi.
#
# args - a list containing the arguments presented as name and value
#        pairs. Valid element names include "dbname", "user", "password",
#        "host", "hostaddr", and "port".
# dbObj - a binding object for a given dbms (e. g. PgSQL() for postgreSQL).
#
# Copyright 2002 J. Zhang, all rights reserved
#
querySAGE <- function (args, dbObj = PgSQL()){

    on.exit(end())
    text1 <- paste("Select a table to query (database = ",
                                     args[["dbname"]], ").", sep = "")
    text2 <- "Select columns from the table"

    columns <- NULL
    colIndex <- NULL
    indexInSel <- NULL

    end <- function(){
        tkdestroy(base)
    }

    # When a table name in the list gets clicked, the name is shown in
    # the entry box besides the list box and all the column names of
    # the table shown in the list box below for selection
    tableSelected <- function(){
        tName <- tclvalue(tkget(tableView, tkcurselection(tableView)))
        writeList(tableName, tName)
        writeList(colView, names(getColumnNames(tName, args, dbObj)))
    }

    # When a user double clicks a column name in the list box, put the
    # library name in the vector for selected SAGE libraries.
    colDClicked <- function(){
        column <- as.character(tkget(colView,(tkcurselection(colView))))
        columns <<- unique(c(columns, column))
        writeList(selectView, columns)
        tkconfigure(clearBut, state = "normal")
        tkconfigure(selectBut, state = "disabled")
    }
    # When a user single clicked a column name, remember that name and
    # activate the select button
    colSClicked <- function(){
        colIndex <<- unlist(strsplit(tkcurselection(colView), " "))
        tkconfigure(selectBut, state = "normal")
    }
    # When a user click the select button, put the selected column name
    # in the list box for selected columns
    selectCol <- function(){
        for(i in colIndex){
            columns <<- c(columns, paste(as.character(tkget(tableName)),
                          ".", as.character(tkget(colView, i)), sep = ""))
        }
        writeList(selectView, unique(columns))
        tkconfigure(selectBut, state = "disabled")
        tkconfigure(clearBut, state = "normal")
    }
    # Remember the column name when a name in the selected column names list
    # box is single clicked
    selSClick <- function(){
        indexInSel <<- unlist(strsplit(tkcurselection(selectView), " "))
        tkconfigure(remBut, state = "normal")
    }
    # Removes the selected column name from the list box for selected
    # column names
    dropSelCol <- function(){
        for(i in indexInSel){
            columns <<- columns[columns != as.character(tkget(selectView, i))]
        }
        writeList(selectView, columns)
        tkconfigure(remBut, state = "disabled")
    }
    # Remove everything from the list box for selected column names
    clearSelCol <- function(){
        writeList(selectView, "")
        columns <<- NULL
        tkconfigure(clearBut, state = "disabled")
    }

    ## Set up the interface
    base <- tktoplevel()
    tktitle(base) <- "BioC Table Inputs Widget"

    # List of table names
    topFrame <- tkframe(base)
    text1Label <- tklabel(topFrame, text = text1, font = "Helvetica 12")
    tkgrid(text1Label, columnspan = 2, pady = 2)
    tableFrame <- tkframe(topFrame)
    tableView <- makeViewer(tableFrame, vWidth = 40, vHeight = 2,
                           hScroll = TRUE, what = "list")
    tkbind(tableView, "<B1-ButtonRelease>", tableSelected)
    nameFrame <- tkframe(topFrame)
    nameLabel <- tklabel(nameFrame, text = "Table selected")
    tableName <- tkentry(nameFrame, width = 40)
    tkpack(nameLabel, tableName)
    tkgrid(tableFrame, nameFrame, padx = 2)
    tkpack(topFrame, pady = 2)

    # Lists for column names
    midFrame <- tkframe(base)
    text2Label <- tklabel(midFrame, text = text2, font = "Helvetica 12")
    tkgrid(text2Label, columnspan = 2, pady = 2)
    # Label for available SAGE libs
    leftFrame <- tkframe(midFrame)
    label1 <- tklabel(leftFrame, text = "Columns in table",
                        font = "Helvetica 11")
    tkpack(label1)
    # List box showing the available SAGE libs
    colFrame <- tkframe(leftFrame)
    colView <- makeViewer(colFrame, vWidth = 40, vHeight = 15,
                           hScroll = TRUE)
    tkbind(colView, "<Double-Button-1>", colDClicked)
    tkbind(colView, "<B1-ButtonRelease>", colSClicked)
    tkpack(colFrame, padx = 5)
    selectBut <- tkbutton(leftFrame, text = "Select >>", width = 12,
		      state = "disabled", command = selectCol)
    tkpack(selectBut)
    tkconfigure(colView, selectmode = "extended")
    # Put the list box for selected SAGE libs and the associated buttons
    rightFrame <- tkframe(base)
    label2 <- tklabel(rightFrame, text = "Columns selected",
                        font = "Helvetica 11")
    tkgrid(label2, columnspan = 2)
    selLFrame <- tkframe(rightFrame)
    selectView <- makeViewer(selLFrame, vWidth = 40, vHeight = 15,
                             hScroll = TRUE)
    tkgrid(selLFrame, columnspan = 2, padx = 5)
    tkconfigure(selectView, selectmode = "extended")
    tkbind(selectView, "<B1-ButtonRelease>", selSClick)
    remBut <- tkbutton(rightFrame, text = "<< Remove", width = 12,
		      state = "disabled", command = dropSelCol)
    clearBut <- tkbutton(rightFrame, text = "Clear", width = 12,
		      state = "disabled", command = clearSelCol)
    tkgrid(remBut, clearBut)
    tkgrid.configure(remBut, sticky = "e")
    tkgrid.configure(clearBut, sticky = "w")

    tkgrid(leftFrame, rightFrame)
    tkpack(midFrame)

    # A text box for filters
    botFrame <- tkframe(base)
    filterLabel <- tklabel(botFrame, text = "Filter: ")
    filterTextFrame <- tkframe(botFrame)
    filterText <- makeViewer(filterTextFrame, vWidth = 70, vHeight = 2,
                           hScroll = FALSE, what = "text")
    tkpack(filterLabel, side = "left", anchor = "ne")
    tkpack(filterTextFrame, side = "left")
    tkpack(botFrame, pady = 2)

    # Put the end button
    endBut <- tkbutton(base, text = "Finish", width = 12, command = end)
    tkpack(endBut, pady = 5)

    writeList(tableView, unlist(getTableNames(args, dbObj)))

    tkwait.window(base)

    return(columns)
}

getTableNames <- function(args, dbObj){
    tableSQL <- "SELECT relname FROM pg_class WHERE relname !~'^pg_'"
    return(executeQuery(tableSQL, args, dbObj))
}

getColumnNames <- function(tableName, args, dbObj){
    columnSQL <- paste("SELECT * FROM ", tableName, " limit 1", sep = "")
    return(executeQuery(columnSQL, args, dbObj))
}










