#selectLibs <- function(){
#    checkMe <- getSLyzerArgs()$dbArgs
#    if(missing(checkMe)){
#        showDBError()
#    }else{
#        libs <- queryInfoDB(infoCol = "filename")
#        picked <- pickItems(libs)
#        libs <- libs[match(picked, libs)]
#        modSLyzerArgs("libs", names(libs))
#    }
#}

ENV <- new.env(hash = TRUE)
getDBArgs <- function(){

    switch(.Platform$OS.type,
           "windows" = dbArgs <- getWinDBArgs(),
           "unix" = dbArgs <- getUnixDBArgs(),
           stop("Unknown operating system"))

    if(!is.null(dbArgs)){
        if(.Platform$OS.type == "unix"){
            if(is.null(dbArgs[["dbname"]]) || is.na(dbArgs[["dbname"]]) ||
               dbArgs[["dbname"]] == ""){
                temp <- tkmessageBox(title = "DB error",
                                     message = paste("Database name",
                                     "can not be null, na, or empty!"),
                                     icon = "warning", type = "ok")
                getDBArgs()
            }else{

                modSLyzerArgs("dbArgs", dbArgs)
            }
        }else{
            if(is.null(dbArgs[["DSN"]]) || is.na(dbArgs[["DSN"]]) ||
               dbArgs[["DSN"]] == ""){
                temp <- tkmessageBox(title = "DB error",
                                     message = paste("Database name",
                                     "can not be null, na, or empty!"),
                                     icon = "warning", type = "ok")
                getDBArgs()
            }else{
                modSLyzerArgs("dbArgs", dbArgs)
            }
        }
        options(show.error.messages = FALSE)
        tryMe <- try(makeConnection(dbArgs))
        options(show.error.messages = FALSE)
        if(inherits(tryMe, "try-error")){
            temp <- tkmessageBox(title = "DB error",
                                     message = paste("Connect connect",
                                     "to database because of", tryMe),
                                     icon = "warning", type = "ok")
            return(FALSE)
        }else{
            closeConn(tryMe)
            return(TRUE)
        }
    }
    return(FALSE)
}

getUnixDBArgs <- function(binding = "pg"){
    args <- list(dbObj = getBinding(binding))

    SAGELyzerArgs <- getSLyzerArgs()
    if(length(SAGELyzerArgs$dbArgs) == 1){
        temp <- argsWidget(list(Database = "", User = "",
                Password = "", Host = "localhost", Counts = "sagecounts",
                Info = "sageinfo", Map = "tagugmap"),
                defaultNames = c("Connect", "Cancel"),
                           inst = paste("Fill in blanks and use default",
                           "values for others if not sure"))
    }else{
        temp <- argsWidget(list(Database = ifelse(
                                is.null(SAGELyzerArgs$dbArgs$dbname), "",
                                SAGELyzerArgs$dbArgs$dbname),
                                User = ifelse(
                                is.null(SAGELyzerArgs$dbArgs$user), "",
                                SAGELyzerArgs$dbArgs$user),
                                Password = ifelse(
                                is.null(SAGELyzerArgs$dbArgs$password), "",
                                SAGELyzerArgs$dbArgs$password),
                                Host = ifelse(
                                is.null(SAGELyzerArgs$dbArgs$host), "",
                                        SAGELyzerArgs$dbArgs$host),
                                Counts = ifelse(
                                is.null(SAGELyzerArgs$dbArgs$counts), "",
                                        SAGELyzerArgs$dbArgs$counts),
                                Info = ifelse(
                                is.null(SAGELyzerArgs$dbArgs$info), "",
                                        SAGELyzerArgs$dbArgs$info),
                                Map = ifelse(
                                is.null(SAGELyzerArgs$dbArgs$map), "",
                                        SAGELyzerArgs$dbArgs$map)),
                                defaultNames = c("Connect", "Cancel"),
                            inst = paste("Fill in blanks and use default",
                           "values for others if not sure"))
    }
    if(!is.null(temp)){
        args[["dbname"]] <- temp[["Database"]]
        args[["user"]] <- temp[["User"]]
        args[["password"]] <- temp[["Password"]]
        args[["host"]] <- temp[["Host"]]
        args[["counts"]] <- temp[["Counts"]]
        args[["info"]] <- temp[["Info"]]
        args[["map"]] <- temp[["Map"]]
        return(args)
    }else{
        return(temp)
    }
}

# Get the correct binding. Only Postgres for now
getBinding <- function(binding = c("pg")){
    binding <- match.arg(binding)
    switch(binding,
           pg = return(PgSQL()),
           stop("Unknown argument for binding"))
}

getWinDBArgs <- function(){
    SAGELyzerArgs <- getSLyzerArgs()
    if(length(SAGELyzerArgs$dbArgs) == 1){
        return(argsWidget(list(DSN = "", counts = "sagecounts",
                               info = "sageinfo", map = "tagugmap"),
                          defaultNames = c("Connect", "Cancel"),
                          inst = paste("Fill in blanks and use default",
                          "values for others if not sure")))
    }else{
        return(argsWidget(list(DSN = ifelse(
                               is.null(SAGELyzerArgs$dbArgs["DSN"]), "",
                               SAGELyzerArgs$dbArgs["DSN"]),
                               counts = ifelse(
                               is.null(SAGELyzerArgs$dbArgs["counts"]), "",
                               SAGELyzerArgs$dbArgs["counts"]),
                               info = ifelse(
                               is.null(SAGELyzerArgs$dbArgs["info"]), "",
                               SAGELyzerArgs$dbArgs["info"]),
                               map = ifelse(
                               is.null(SAGELyzerArgs$dbArgs["map"]), "",
                               SAGELyzerArgs$dbArgs["map"])),
                          defaultNames = c("Connect", "Cancel"),
                           inst = paste("Fill in blanks and use default",
                           "values for others if not sure")))
    }
}

getTag <- function(){
    cancel <- FALSE
    temp <- toupper(unlist(argsWidget(list(TargetSAGE = "")),
                               use.names = FALSE))
    while(nchar(temp) != 10){
        if(tclvalue(tkmessageBox(
                     message = "Target must be 10 letters.\n Re-enter?",
                     icon = "error", type = "yesno")) == "yes"){
            temp <- toupper(unlist(argsWidget(list(TargetSAGE = "")),
                               use.names = FALSE))
        }else{
            cancel <- TRUE
            break
        }
    }
    if(!cancel){
        modSLyzerArgs("targetSAGE", temp)
    }
}

setKNNArgs <- function(){
    checkMe <- getSLyzerArgs()$dbArgs
    if(missing(checkMe)){
        showDBError()
    }else{
        args <- KNNArgsWidget()
        if(!is.null(args)){
            SAGELyzerArgs <- getSLyzerArgs()
            SAGELyzerArgs[["targetSAGE"]] <-  args[["targetSAGE"]]
            SAGELyzerArgs[["libs"]] <-  args[["libs"]]
            for(i in  c("normalize", "k", "dist", "trans")){
                SAGELyzerArgs[[i]] <- args[[i]]
            }
            writeSLyzerArgs(SAGELyzerArgs)
        }
    }
}

#getDefaults <- function(){
#    formalArgs <- formals(SAGELyzer)
#    argsNames <- c("tagColName", "normalize", "k", "dist", "trans")
#    dArgs <- argsWidget(list(TagName = formalArgs[["tagColName"]],
#                                normalize = formalArgs[["normalize"]],
#                                k = formalArgs[["k"]],
#                                distance = formalArgs[["dist"]],
#                                transform = formalArgs[["trans"]]),
#                        defaultNames = c("Set", "Cancel"))
#    SAGELyzerArgs <- getSLyzerArgs()
#    for(i in 1:length(dArgs)){
#        SAGELyzerArgs[[argsNames[[i]]]] <- dArgs[[i]]
#    }
#    writeSLyzerArgs(SAGELyzerArgs)
#}

getSLyzerArgs <- function(argName = "SAGELyzerArgs"){
    return(get(argName, ENV))
}

writeSLyzerArgs <- function(args, argName = "SAGELyzerArgs"){
    assign(argName, args, ENV)
}

modSLyzerArgs <- function(argName, value){
    SAGELyzerArgs <- getSLyzerArgs()
    SAGELyzerArgs[[argName]] <- value
    writeSLyzerArgs(SAGELyzerArgs)
}

setSLyzerArgs <- function(){
    writeSLyzerArgs(formals(SAGELyzer))
}

runSLyzer <- function(){
    # Just to make sure the status bar gets updated
    Sys.sleep(1)
    if(getSLyzerArgs()$targetSAGE == ""){
        tkmessageBox(title = "KNN Error",
                     message = paste("No target tag has been set yet.\n",
                                     "Use \"Set arguments\" to select",
                     "one"), icon = "warning", type = "ok")
        return(invisible())
    }else{
        args <- getSLyzerArgs()
        # This is not working although args is a list while the system
        # says no
        # knn <- do.call("SAGELyzer", args)

        # Do it this way instead
        knn <- SAGELyzer(dbArgs = args[["dbArgs"]],
                     targetSAGE = args[["targetSAGE"]],
                     libs = args[["libs"]], normalize = args[["normalize"]],
                     tagColName = args[["tagColName"]],
                     k = as.numeric(args[["k"]]),
                     dist = args[["dist"]], trans = args[["trans"]])
        if(!is.null(knn)){
            writeSAGEKNN(knn, args[["targetSAGE"]])
            tempData <- matrix(c(names(knn), knn), ncol = 2, byrow = FALSE)
            colnames(tempData) <- c("Tag", "Distance")
            dataViewer(tempData, paste("Target SAGE:", args[["targetSAGE"]]))
        }
    }
}

writeSAGEKNN <- function(knn, targetSAGE){
    assign("SAGEKNN", list(targetSAGE = targetSAGE, knn = knn),
           "package:SAGElyzer")
}

getSAGEKNN <- function(){
    knn <- get("SAGEKNN","package:SAGElyzer")
    return(knn)
}

getLibCounts <- function(){
    # Just to make sure the status bar gets updated
    Sys.sleep(1)
    options(show.error.messages = FALSE)
    knn <- try(getSAGEKNN())
    options(show.error.messages = FALSE)
    if(inherits(knn, "try-error")){
        tkmessageBox(title = "KNN Error",
                     message = paste("No counts are available.\n",
                     "You have not run knn yet"),
                     icon = "warning", type = "ok")
        return(invisible())
    }else{
        args <- getSLyzerArgs()

        if(is.na(match("*", args[["libs"]]))){
            args[["libs"]] <- c(args[["tagColName"]], args[["libs"]])
        }

        countsSQL <- paste("select ", paste(args[["libs"]], sep = "",
                                       collapse = ","), " from ",
                       args[["dbArgs"]][["counts"]], " where ",
                       args[["tagColName"]], " in('",
                       paste(names(knn$knn), sep = "", collapse = "','"),
                       "')", sep = "")
        conn <- makeConnection(args[["dbArgs"]])
        on.exit(closeConn(conn))
#        print("Querying the database")
        counts <- as.matrix(executeQuery(countsSQL, conn))
        # Make sure the target will be the firest row
        target <- getTargetRow(args[["dbArgs"]], conn,
                               setdiff(args[["libs"]], args[["tagColName"]]),
                               args[["tagColName"]], args[["targetSAGE"]])
        # Convert library code back to library name
        dataToShow <- rbind(as.matrix(target), counts)
        libs <- queryInfoDB(infoCol = "filename")
        colnames(dataToShow) <- as.vector(c("Tag", libs[
                     match(colnames(dataToShow)[2:ncol(dataToShow)],
                           names(libs))]))
        dataViewer(dataToShow)
    }
}

mapLib2File <- function(){
    checkMe <- getSLyzerArgs()$dbArgs
    if(missing(checkMe)){
        showDBError()
    }else{
        libs <- queryInfoDB(infoCol = "filename")
        libs <- cbind(names(libs), as.vector(libs))
        colnames(libs) <- c("DB Column Name", "File Name")
        dataViewer(cbind(names(libs), libs), save = FALSE)
    }
}

linkTag2UG <- function(){
    # Make sure the status bar gets updated
    Sys.sleep(1)
    options(show.error.messages = FALSE)
    knn <- try(getSAGEKNN()$knn)
    options(show.error.messages = FALSE)
    if(inherits(knn, "try-error")){
        tkmessageBox(title = "KNN Error",
                     message = paste("There is nothing to map.\n",
                     "You have not run knn yet"),
                     icon = "warning", type = "ok")
    }else{
        args <- getSLyzerArgs()
        linkSQL <- paste("select * from ", args[["dbArgs"]][["map"]],
                     " where ", args[["tagColName"]] , " in('",
                     paste(c(args[["targetSAGE"]],
                             names(knn)), sep = "",
                           collapse = "','"), "')", sep = "")
        conn <- makeConnection(args[["dbArgs"]])
        on.exit(closeConn(conn))
        mappings <- remapTagNUG(as.matrix(executeQuery(linkSQL, conn)))
        # closeConn(conn)
        url <- file.path(.path.package("SAGElyzer"), "temp", "tagUGMap.html")
        ll.htmlpage(mappings[,2], url, table.head = "UniGene ID",
                    othernames = mappings[,1],
                    title = paste("Tag-UniGene Mappings(Target = ",
                    args[["targetSAGE"]], ")", sep = ""), repository = "ug")

        browseURL(url)
    }
}
remapTagNUG <- function(mappings){
    equalized <- NULL
    equalATag <- function(row){
        ugs <- unlist(strsplit(mappings[row, 2], ";"),
                      use.names = FALSE)
        temp <- cbind(rep(mappings[row, 1], length(ugs)), ugs)
        if(is.null(equalized)){
            equalized <<- temp
        }else{
            equalized <<- rbind(equalized, temp)
        }
    }
    sapply(1:nrow(mappings), equalATag)
    return(equalized)
}

SAGEFromGEO <- function(){
    args <- GEOSAGEWidget()
    if(!is.null(args)){
        do.call("getGEOSAGE", args)
        return("Done")
    }else{
        return("Aborted")
    }
}

procSAGE <- function(){
    checkMe <- getSLyzerArgs()$dbArgs
    if(missing(checkMe)){
        showDBError()
    }else{
        args <- mergeSAGEWidget()
        # Pause to let the system redraw the screen
        Sys.sleep(1)
        if(!is.null(args)){
            merged <- do.call("mergeSAGE", args)
            return("Done")
        }else{
            return("Aborted")
        }
    }
}

mapSAGE2UG <- function(){
    checkMe <- getSLyzerArgs()$dbArgs
    if(missing(checkMe)){
        showDBError()
    }else{
        args <- mapSAGEWidget()
        if(!is.null(args)){
            if(.Platform$OS.type == "windows"){
                args[["fromWeb"]] <- FALSE
            }
            do.call("SAGEMapper", args)
            return("Done")
        }else{
            return("Aborted")
        }
    }
}

showDBError <- function(){
    tkmessageBox(title = "DB error", message = paste("You need to make a",
                 "DB connection using DBConnecter!"), icon = "warning",
                 type = "ok")
    return(invisible())
}

findNG4Tag <- function(){
    org <- argsWidget(list(Organism = "human"),
                      inst = paste("Enter an organism name"))
    if(is.null(org)){
        return(invisible())
    }
    if(!any(tolower(org) == c("human", "mouse", "rat"))){
         tkmessageBox(title = "Error",
                     message = paste("Organism name has to be either",
                     "human, mouse, or rat"),
                     icon = "warning", type = "ok")
    }else{
        Sys.sleep(1)
        options(show.error.messages = FALSE)
        knn <- try(getSAGEKNN()$knn)
        options(show.error.messages = FALSE)
        if(inherits(knn, "try-error")){
            tkmessageBox(title = "KNN Error",
                         message = paste("No target tags available.\n",
                         "You have not run knn yet"),
                         icon = "warning", type = "ok")
        }else{
            args <- getSLyzerArgs()
            linkSQL <- paste("select * from ", args[["dbArgs"]][["map"]],
                             " where ", args[["tagColName"]] , " in('",
                             paste(c(args[["targetSAGE"]],
                                     names(knn)), sep = "",
                                   collapse = "','"), "')", sep = "")
            conn <- makeConnection(args[["dbArgs"]])
            on.exit(closeConn(conn))
            mappings <- remapTagNUG(as.matrix(executeQuery(linkSQL, conn)))
            colnames(mappings) <- c("tag", "unigene")
            neighbors <- neighborGeneFinder(mappings, "unigene", org)
        }
    }
}
