# This function takes a file name for the merged SAGE libraries data file and
# writes the data into a table in a database.

writeSAGE2DB <- function(dbArgs, colNames, keys = "", numCols = "",
                         fileName, what = c("counts", "map", "info"),
                         charNum = 20, type = "int4"){

    what <- match.arg(what)
    conn <- makeConnection(dbArgs)

    if(tableExists(conn, dbArgs[[what]])){
        executeQuery(paste("drop table", dbArgs[[what]]), conn, TRUE)
    }

    # Create an empty table
    newTableSQL <- paste("create table ", dbArgs[[what]], " ( ",
                         getColSQL(colNames, charNum, keys, numCols, type),
                         ifelse(keys == "", "",
                         paste(", primary key(", paste(keys, sep = "",
                         collapse = ","), ")", sep = "")), ")", sep = "")

    executeQuery(newTableSQL, conn, TRUE)
    if(.Platform$OS.type == "unix"){
        # Copy file to the table created
        writeSQL <- paste("copy ", dbArgs[[what]], " from '", fileName,
                          "' using delimiters '\t'", sep = "")
        executeQuery(writeSQL, conn, TRUE)
    }else{
        temp <- read.table(fileName, sep = "\t", header = FALSE,
                           as.is = TRUE)
        colnames(temp) <- colNames
        sqlSave(conn, temp, dbArgs[[what]], TRUE, FALSE, FALSE)
    }
#    closeConn(conn)
}

# Returns part of a SQL statement that set up the clumns of new table.
getColSQL <- function(colNames, charNum, keys, numCols, type){
    colSQL <- NULL
    for(i in colNames){
        if(any(keys == i)){
            toAdd <- "not null"
        }else{
            toAdd <- ""
        }
        if(any(numCols == i)){
            if(length(type) == 1){
                toAdd <- paste(type, "default 0")
            }else{
                toAdd <- paste(type[i], "default 0")
            }
        }else{
            if(length(charNum) == 1){
                temp <- charNum[1]
            }else{
                temp <- charNum[i]
            }
            toAdd <- paste("varchar(", temp, ") ", toAdd, sep = "")
        }
        if(is.null(colSQL)){
            colSQL <- paste(tolower(i), " ", toAdd, sep = "")
        }else{
            colSQL <- paste(colSQL, paste(tolower(i), " ", toAdd,
                                              sep = ""), sep = ",")
        }
    }
    return(colSQL)
}

