\name{getNormFactor}
\alias{getNormFactor}
\alias{queryInfoDB}
\title{Functions that get normalization factors for SAGE libraries}
\description{
  SAGE libraries vary in the total number of tags so that counts need to
  be normalized across libraries. These functions get the normalization
  factors that are stored in a database table. 
}
\usage{
getNormFactor(normalize = c("min", "max", "none"), libs)
queryInfoDB(libCol = "libname",
                        infoCol = c("filename", "minfactor", "maxfactor"))
}
\arguments{
  \item{normalize}{\code{normalize} a character string for the means of
    normalization. Have to be either "min", "max", or "none"}
  \item{libs}{\code{libs} a vector of character strings for the names of
    SAGE libraries to be normalized}
  \item{libCol}{\code{libCol} a character string for the name of the
    column in a database table where names of SAGE libraries are stored}
  \item{infoCol}{\code{infoCol} a vector of character strings for the
    names of database columns where SAGE library information is kept}
}
\details{
  The normalization factor is calculated by dividing the total number of
  tags for a given library by the maximum or minimum value across the
  library.

  \code{\link{getNormFactor}} returns the normalization factors for a
  given set of SAGE libraries.

  \code{\link{queryInfoDB}} queries a database table containing
  information about SAGE libries to get the normalization factor for
  SAGE libraries.
}
\value{
  Both \code{\link{getNormFactor}} and  \code{\link{queryInfoDB}} return
  a data frame containing normalization factors for a set of SAGE libraries.
}
\references{\url{http://www.ncbi.nlm.nih.gov/SAGE/} }
\author{Jianhua Zhang }
\note{The functions are part of the Bioconductor project at Dana-Farber
  Cancer Institute to provide Bioinformatics functionalities through R}

\seealso{\code{\link{SAGELyzer}}}
\examples{
# No example is given as database support is required
}
\keyword{misc}
